Attribute VB_Name = "CommandsDB"
Option Explicit

Public Type ParameterInfo
  Name As String
  AllowedTytes As enmSegmentType
  VariableType As Long '-1 means simple var
End Type

Public Enum enmCommandUsage
  cuPrimaryCommand = 1
  cuParameter = 2
End Enum

Public Type CommandParams
  NumNames As Long
  Names() As String
  MinParameterCount As Long
  MaxParameterCount As Long
  PrametersInfo() As ParameterInfo
  Description As String
  ParserLevel1Only As Boolean
  DoNotExecute As Boolean
  Usage As enmCommandUsage
End Type

Public CommandParams() As CommandParams
Public Const NumNativeCommands As Long = cmdTerminate

Public Sub Init()
Dim W As enmCommand

ReDim CommandParams(1 To NumNativeCommands)

For W = 1 To NumNativeCommands
  With CommandParams(W)
    Select Case W
    Case cmdTo
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "To"
      
      .MinParameterCount = 1
      .MaxParameterCount = &H7FFFFFF
      .DoNotExecute = True
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "ProcedureName"
      .PrametersInfo(0).AllowedTytes = ptProcedure
      .PrametersInfo(1).Name = "Parameters"
      .PrametersInfo(1).AllowedTytes = 0
      .Description = "Procedure declaration"
      .ParserLevel1Only = True
      
    Case cmdDim
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Dim"
      
      .MinParameterCount = 1
      .MaxParameterCount = &H7FFFFFFF
      .DoNotExecute = True
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0)
      .PrametersInfo(0).Name = "VariableNames"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .Description = "Variable declaration"
      .ParserLevel1Only = True
      
    Case cmdConst
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Const"
      
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .DoNotExecute = True
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "ConstantName"
      .PrametersInfo(0).AllowedTytes = ptConstant
      .PrametersInfo(1).Name = "ConstantValue"
      .PrametersInfo(1).AllowedTytes = ptConstant
      .Description = "Constant declaration"
      .ParserLevel1Only = True
    
    Case cmdStruct
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Struct"
      
      .MinParameterCount = 2
      .MaxParameterCount = &H7FFFFFF
      .DoNotExecute = True
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "StructName"
      .PrametersInfo(1).Name = "Members..."
      .Description = "Define a Struct (Also known as Record or UDT)"
      .ParserLevel1Only = True
    
    Case cmdEnd
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "End"
      
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .DoNotExecute = True
      .Usage = cuPrimaryCommand
      .Description = "End of procedure"
      
    Case cmdCall
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Call"

      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "ProcedureName"
      .PrametersInfo(0).AllowedTytes = ptProcedure
      .Description = "Executes a procedure"
    Case cmdRepeat
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Repeat"
      
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "RepeatCount"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Repeats the following code <RepeatCount> times" & vbCrLf & "Repeat block ends with 'Loop' or 'EndRepeat'"
      
    Case cmdWhile
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "While"
      
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Condition"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Repeats the following code as long as <Condition> is true (non-zero)" & vbCrLf & "While block ends with 'Loop'"
      
    Case cmdUntil
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Until"
      
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Condition"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Repeats the following code until <Condition> is true (non-zero)" & vbCrLf & "Until block ends with 'Loop'"
      
    Case cmdLoop
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Loop"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .DoNotExecute = True
      .Description = "End of a Repeat or a While loop"
      
    Case cmdElse
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "Else"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .DoNotExecute = True
      .Description = "Code below is executed when the if condition above is false"
      
    Case cmdEndIf
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "EndIf"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .DoNotExecute = True
      .Description = "End of an if block"
      
    Case cmdIf
      .NumNames = 1
      ReDim .Names(0)
      .Names(0) = "If"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Condition"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Executes the code blow if <Condition> is true (non-zero)"
      
    Case cmdBK
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Back"
      .Names(1) = "BK"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Distance"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Moves Turtle backward"
      
    Case cmdFD
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Forward"
      .Names(1) = "FD"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Distance"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Moves Turtle forward"
      
    Case cmdRT
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Right"
      .Names(1) = "RT"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Angle"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Turn Turtle right (relative to current angle)"
      
    Case cmdLT
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Left"
      .Names(1) = "LT"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Angle"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Turn Turtle left (relative to current angle)"
      
    Case cmdSetTurtleAngle
      .NumNames = 3
      ReDim .Names(0 To 2)
      .Names(0) = "SetTurtleAngle"
      .Names(1) = "SetAngle"
      .Names(2) = "SetH"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Angle"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets Turtle rotation angle (absolute)"
      
    Case cmdSetTurtleX
      .NumNames = 3
      ReDim .Names(0 To 2)
      .Names(0) = "SetTurtleX"
      .Names(1) = "SetX"
      .Names(2) = "SX"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Position"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets Turtle absolute X Coordinate"
      
    Case cmdSetTurtleY
      .NumNames = 3
      ReDim .Names(0 To 2)
      .Names(0) = "SetTurtleY"
      .Names(1) = "SetY"
      .Names(2) = "SY"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Position"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets Turtle absolute Y Coordinate"
      
      
    Case cmdGetTurtleAngle
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "GetTurtleAngle"
      .Names(1) = "GetAngle"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuParameter
      .Description = "Returns Turtle rotation angle (absolute)"
      
    Case cmdGetTurtleX
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "GetTurtleX"
      .Names(1) = "GetX"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuParameter
      .Description = "Returns Turtle absolute X Coordinate"
      
    Case cmdGetTurtleY
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "GetTurtleY"
      .Names(1) = "GetY"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuParameter
      .Description = "Returns Turtle absolute Y Coordinate"
      
    Case cmdWait
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Wait"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Milliseconds"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Suspends the program"
      
    Case cmdMoveDelay
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "MoveDelay"
      .Names(1) = "MD"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Milliseconds"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the time the program will wait after every move or turn command"
      
    Case cmdSetTurtlePos
      .NumNames = 4
      ReDim .Names(0 To 3)
      .Names(0) = "SetTurtlePos"
      .Names(1) = "SetPos"
      .Names(2) = "SetXY"
      .Names(3) = "SXY"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "X"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Y"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Sets Turtle absolute X and Y Coordinate"
      
    Case cmdPU
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PenUp"
      .Names(1) = "PU"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Moves Turtles virtual pen up (moving without drawing)"
      
    Case cmdPD
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PenDown"
      .Names(1) = "PD"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Moves Turtles virtual pen down (moving with drawing)"

    Case cmdST
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "ShowTurtle"
      .Names(1) = "ST"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Shows the Turtle if hidden"

    Case cmdHT
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "HideTurtle"
      .Names(1) = "HT"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Hides the Turtle if visible"
    
    Case cmdSetDrawWidth
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "SetDrawWidth"
      .Names(1) = "DrawWidth"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Width"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the thickness of the lines beeing drawn" & vbCrLf & "NOTE: Has no effect when alphablend is used"
      
    Case cmdMake
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Make"
      .Names(1) = "Set"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Variable"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Sets <Variable> to the given value"

    Case cmdAdd
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Add"
      .Names(1) = "Sum"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Variable"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Adds a value to a variable"
      
    Case cmdSub
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Sub"
      .Names(1) = "Substract"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Variable"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Substracts a value from a variable"

    Case cmdMul
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Mul"
      .Names(1) = "Multiply"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Variable"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Multiplies a variable with a value"

    Case cmdDiv
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Div"
      .Names(1) = "Divide"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Variable"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Divides a variable by a value"

    Case cmdRnd
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Rnd"
      .Names(1) = "Random"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Generates a random number from zero to <Value>"

    Case cmdSin
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Sin"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the sine of an angle"

    Case cmdCos
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Cos"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the cosine of an angle"
    
    Case cmdTan
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Tan"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the tangent of an angle"
      
    Case cmdATan
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "ATan"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the arctangent of a value"
      
    Case cmdASin
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "ASin"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the arcsine of a value"
      
    Case cmdACos
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "ACos"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Calculates the arccosine of a value"
      
    Case cmdPower
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Power"
      .Names(1) = "Pow"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Base"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Exponent"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Usage = cuParameter
      .Description = "Calculates <Base> raised to the power of <Exponent>"

    Case cmdSqr
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Sqr"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Value"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Usage = cuParameter
      .Description = "Calculates the root of a number"
    
    Case cmdLog
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Log"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Base"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Usage = cuParameter
      .Description = "Calculates the logarithm of <Value>"

    Case cmdLn
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Ln"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Usage = cuParameter
      .Description = "Calculates the natural logarithm of <Value>"
    
    Case cmdExp
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Exp"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Usage = cuParameter
      .Description = "Returns e (the base of natural logarithms) raised to the power of <Value>"
      
    Case cmdRound
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Round"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "NumDigitsAfterDecimal"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Usage = cuParameter
      .Description = "Rounds a number to a specified number of digits"
      
    Case cmdFix
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Fix"
      .Names(1) = "Integer"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Value"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Usage = cuParameter
      .Description = "Returns the integer portion of a number"
      
    Case cmdPC
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PenColor"
      .Names(1) = "PC"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "ColorIndex"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the drawing color to the specified palette-color (0-15)" & vbCrLf & "Use PenColorRGB for all colors"
    
    Case cmdBG
      .NumNames = 3
      ReDim .Names(0 To 2)
      .Names(0) = "Background"
      .Names(1) = "BG"
      .Names(2) = "BackColor"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "ColorIndex"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the background color of the drawing-board to the specified palette-color (0-15)" & vbCrLf & "Use BackColorRGB for all colors"
      
    Case cmdSetAlpha
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "SetAlpha"
      .Names(1) = "SA"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "AlphaAmount"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the alpha transparency of the lines beeing drawed (Use 32Bit system color depth!)"
      
    Case cmdPCRGB
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PenColorRGB"
      .Names(1) = "PCRGB"
      .MinParameterCount = 3
      .MaxParameterCount = 3
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "Red"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Green"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .PrametersInfo(2).Name = "Blue"
      .PrametersInfo(2).AllowedTytes = 0
      .PrametersInfo(2).VariableType = -1
      .Description = "Sets the drawing color to the specified color"
      
    Case cmdBGRGB
      .NumNames = 3
      ReDim .Names(0 To 2)
      .Names(0) = "BackgroundRGB"
      .Names(1) = "BGRGB"
      .Names(2) = "BackColorRGB"
      .MinParameterCount = 3
      .MaxParameterCount = 3
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "Red"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Green"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .PrametersInfo(2).Name = "Blue"
      .PrametersInfo(2).AllowedTytes = 0
      .PrametersInfo(2).VariableType = -1
      .Description = "Sets the background color of the drawing-board to the specified color"
      
    Case cmdFloodFill
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "FloodFill"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Fills the area around Turtle with the current PenColor until it hits a boundary"
      
    Case cmdGetPointColor
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "GetPointColor"
      .MinParameterCount = 3
      .MaxParameterCount = 3
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "Red"
      .PrametersInfo(0).AllowedTytes = ptVariable
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Green"
      .PrametersInfo(1).AllowedTytes = ptVariable
      .PrametersInfo(1).VariableType = -1
      .PrametersInfo(2).Name = "Blue"
      .PrametersInfo(2).AllowedTytes = ptVariable
      .PrametersInfo(2).VariableType = -1
      .Description = "Fills the three Parameter Variables with the Color of the Pixel ""under"" the Turtle"
      
      
    Case cmdClear
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Clear"
      .Names(1) = "Cls"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Clears the drawing-board"
      
    Case cmdHome
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Home"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Moves the Turtle back to its starting position (0,0)"
      
    Case cmdDraw
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Draw"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Resets Trutle and drawing-board"
    
    Case cmdEnableBackBuffer
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "EnableBackBuffer"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Drawings are not automaticaly made visible, use 'Update' to do so (very useful for animations!)"
    
    Case cmdDisableBackBuffer
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "DisableBackBuffer"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Disables drawing backbuffer (see 'EnableBackBuffer')"
    
    Case cmdEnableAutoUpdate
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "EnableAutoUpdate"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Automatically update display after every draw command even if 'EnableBackbuffer' is used"
    
    Case cmdDisableAutoUpdate
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "DisableAutoUpdate"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Disables automatic display update if 'EnableBackbuffer' is used"
      
    Case cmdUpdate
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Update"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Updates the display if a BackBuffer is used (see 'EnableBackbuffer')"
      
    Case cmdMath
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Math"
      .Names(1) = "" '!!!
      .MinParameterCount = 3
      .MaxParameterCount = 3
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "Value1"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Operand"
      .PrametersInfo(1).AllowedTytes = ptOperand
      .PrametersInfo(2).Name = "Value2"
      .PrametersInfo(2).AllowedTytes = 0
      .PrametersInfo(2).VariableType = -1
      .Description = "Calculates the specified mathematical operation (Optional, use brackets without command instead)"
    
    Case cmdDrawText
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "DrawText"
      .Names(1) = "TextOut"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Text"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Draws text to the current Turtle location"
    
    Case cmdPrintConsole
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PrintConsole"
      .Names(1) = "Print"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "Text"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Outputs text to the Console window"
    
    Case cmdSetFont
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "SetFont"
      .MinParameterCount = 3
      .MaxParameterCount = 3
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "Fontname"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Size"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .PrametersInfo(2).Name = "Bold"
      .PrametersInfo(2).AllowedTytes = 0
      .PrametersInfo(2).VariableType = -1
      .Description = "Sets the font of the text beeing drawn with the 'DrawText' command"
    
    Case cmdPenMode
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "PenMode"
      .Names(1) = "SetPenMode"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "PenMode"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the DrawMode of the Trurtle to one of the following Modes:" & vbCrLf & "pmCopyPen (Default); pmInvert; pmNotCopyPen; pmXorPen; pmNotXorPen" & vbCrLf & vbCrLf & "NOTE: Has no effect when alphablend is used"

    Case cmdKeyDown
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "KeyDown"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "VirtualKeyCode"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Returns True if the specified Key has been pressed since the last call of this Function"
      
    Case cmdInkey
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Inkey"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuParameter
      .Description = "Pops the first entry from the keyboard stack and returns it as a character"
      
    Case cmdStrLen
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "StrLen"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "String"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Returns the length of a String"
        
    Case cmdStrLeft
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "StrLeft"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "String"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Length"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Returns a string containing a specified number of characters from the left side of a string"

    Case cmdStrRight
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "StrRight"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "String"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Length"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Returns a string containing a specified number of characters from the right side of a string"
      
    Case cmdStrMid
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "StrMid"
      .MinParameterCount = 2
      .MaxParameterCount = 3
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 2)
      .PrametersInfo(0).Name = "String"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Start"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .PrametersInfo(2).Name = "Length"
      .PrametersInfo(2).AllowedTytes = 0
      .PrametersInfo(2).VariableType = -1
      .Description = "Returns a string containing a specified number of characters from a string."
  
    Case cmdAsc
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Asc"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "String"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Returns the character code corresponding to the first letter in a string"
      
    Case cmdChr
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Chr"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "CharCode"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Returns a String containing the character associated with the specified character code"
      
    Case cmdString
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "String"
      .MinParameterCount = 2
      .MaxParameterCount = 2
      .Usage = cuParameter
      ReDim .PrametersInfo(0 To 1)
      .PrametersInfo(0).Name = "Number"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .PrametersInfo(1).Name = "Character"
      .PrametersInfo(1).AllowedTytes = 0
      .PrametersInfo(1).VariableType = -1
      .Description = "Returns a string containing a repeating character string of the length specified"
      
    Case cmdReturn
      .NumNames = 1
      ReDim .Names(0 To 0)
      .Names(0) = "Return"
      .MinParameterCount = 1
      .MaxParameterCount = 1
      .Usage = cuPrimaryCommand
      ReDim .PrametersInfo(0 To 0)
      .PrametersInfo(0).Name = "ReturnValue"
      .PrametersInfo(0).AllowedTytes = 0
      .PrametersInfo(0).VariableType = -1
      .Description = "Sets the return value of the current procedure"
      
      
    Case cmdExitLoop
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "ExitLoop"
      .Names(1) = "Break"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Exits a loop" & vbCrLf & "NOTE: If this command is used outside a loop the current procedure is canceled"
      
    Case cmdExitProcedure
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "ExitProcedure"
      .Names(1) = "Stop"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Exits the current Procedure"
      
    Case cmdTerminate
      .NumNames = 2
      ReDim .Names(0 To 1)
      .Names(0) = "Terminate"
      .Names(1) = "Quit"
      .MinParameterCount = 0
      .MaxParameterCount = 0
      .Usage = cuPrimaryCommand
      .Description = "Terminates the running program"
      
    End Select
  End With
Next
End Sub


Public Function ParameterTypeAsString(T As enmSegmentType) As String
If T And ptConstant Then ParameterTypeAsString = ParameterTypeAsString & "Constant or "
If T And ptNativeCommand Then ParameterTypeAsString = ParameterTypeAsString & "NativeCommand or "
If T And ptVariable Then ParameterTypeAsString = ParameterTypeAsString & "Variable or "
If T And ptProcedure Then ParameterTypeAsString = ParameterTypeAsString & "Procedure or "
If T And ptOperand Then ParameterTypeAsString = ParameterTypeAsString & "Operand or "
If T And ptBlockIndex Then ParameterTypeAsString = ParameterTypeAsString & "BlockIndex or "

ParameterTypeAsString = Left$(ParameterTypeAsString, Len(ParameterTypeAsString) - 4)
End Function

Public Function GetProcedureInfo(Proc As Long) As CommandParams
Dim W As Long

With GetProcedureInfo
  .MinParameterCount = Program.Procedures(Proc).NumParameters
  .MaxParameterCount = .MinParameterCount
  If .MaxParameterCount Then
    ReDim .PrametersInfo(0 To .MaxParameterCount - 1)
    For W = 0 To .MaxParameterCount - 1
      .PrametersInfo(W).Name = Program.Procedures(Proc).LocalVariablesInfo(W).Name
      .PrametersInfo(W).AllowedTytes = 0
      .PrametersInfo(W).VariableType = Program.Procedures(Proc).LocalVariablesInfo(W).VarType
    Next
  End If
  .Usage = cuParameter Or cuPrimaryCommand
End With
  
End Function
