Attribute VB_Name = "Interpreter"
Option Explicit

Private Type ActiveProcedure
  LocalVariables() As Variant
  ReturnValue As Variant
End Type

Private ActiveProcedures() As ActiveProcedure
Private NumActiveProcedures As Long

Private CurrentLine As Long


Public Sub Run()
Dim InitCommand As Command
Dim result As Variant

Program.State = psRunning
Form1.ProgramStateChanged

Randomize Timer
Erase ActiveProcedures
NumActiveProcedures = 0

On Error GoTo Err

InitCommand.Type = ptProcedure
InitCommand.Index = MainProcedure

If Procedures(MainProcedure).NumParameters Then
  If GetStartParams(InitCommand.Parameters) = False Then
    Program.State = psInactive
    Form1.ProgramStateChanged
    Exit Sub
  End If
End If

Form1.Console.PrintText "Starting execution", 0, ciRun

Call ClearKeyStack

result = ExecuteProcedure(InitCommand, -1) 'Run Program

Program.State = psInactive
Form1.ProgramStateChanged
If Options.DebugMode Then Form1.ProcedureInfoChanged ""

If Program.bCancel Then
  Form1.Console.PrintText "Program canceled", &H80&, ciStop
Else
  Form1.Console.PrintText "Program ended", 0, ciStop
End If

If VarType(result) <> vbEmpty Then
  Form1.Console.PrintText "Program returned " & result, &H8000&, ciResult
End If

Call ClearKeyStack

Exit Sub
Err:
Form1.CodeEditor.SetArrow CurrentLine
Form1.Console.PrintText "Error in line " & CurrentLine, vbRed, ciError
Form1.Console.PrintText Err.Description, vbRed, ciError
'MsgBox Err.Description & vbCrLf & "Line: " & CurrentLine, vbExclamation, "Runtime Error"
Program.State = psInactive
Form1.ProgramStateChanged
If Options.DebugMode Then Form1.ProcedureInfoChanged ""

Form1.CodeEditor.SelectLine CurrentLine
End Sub

Private Function ExecuteProcedure(Command As Command, CurrentProcedure As Long) As Variant
Dim W As Long
Dim LocalVars() As Variant

With Procedures(Command.Index)
  
  If .NumLocalVariables Then
    'Initialisize Local Variables
    ReDim LocalVars(0 To .NumLocalVariables - 1)
    'Get Procedure Parameters
    For W = 0 To .NumParameters - 1
      LocalVars(W) = GetParameterValue(Command.Parameters(W), CurrentProcedure)
    Next
  End If
  
  'Enlarge Procedure Variables Stack
  ReDim Preserve ActiveProcedures(0 To NumActiveProcedures)
  NumActiveProcedures = NumActiveProcedures + 1
  'Put Local Variables on the Stack
  If .NumLocalVariables Then ActiveProcedures(NumActiveProcedures - 1).LocalVariables = LocalVars
  
  'Debug
  If Options.DebugMode Then
    If Options.ShowProcedureDetailsWhileRunning Then
      Form1.ProcedureInfoChanged .Name, NumActiveProcedures
    End If
  End If

End With

'Execute Procedure Code
Call ExecuteBlock(Command.Index, 0)
NumActiveProcedures = NumActiveProcedures - 1

'Debug
If Options.DebugMode Then
  If Options.ShowProcedureDetailsWhileRunning Then
    If CurrentProcedure <> -1 Then
      Form1.ProcedureInfoChanged Procedures(CurrentProcedure).Name, NumActiveProcedures
    Else
      Form1.ProcedureInfoChanged "", NumActiveProcedures
    End If
  End If
End If

'Get Return Value
ExecuteProcedure = ActiveProcedures(NumActiveProcedures).ReturnValue

'Delete Procedure from Stack
If NumActiveProcedures Then
  ReDim Preserve ActiveProcedures(0 To NumActiveProcedures - 1)
Else
  Erase ActiveProcedures
End If

If bCancel Then If CancelReason < crProgramExit Then bCancel = False: CancelReason = 0

End Function

Private Sub ExecuteBlock(Procedure As Long, bLock As Long)
Dim W As Long

With Procedures(Procedure).Blocks(bLock)
  For W = 0 To .NumCodeLines - 1
    CurrentLine = .CodeLines(W).TextCodeLine
    If Options.DebugMode Then If ShowCurrentLineWhileRunning Then Form1.CodeEditor.SetArrow CurrentLine
    
    If Program.Commands(.CodeLines(W).PrimaryCommand).Type = ptNativeCommand Then
      Call ExecCommand(Program.Commands(.CodeLines(W).PrimaryCommand), Procedure)
    Else
      Call ExecuteProcedure(Program.Commands(.CodeLines(W).PrimaryCommand), Procedure)
    End If
    
    If Options.DebugMode Then
      If ShowCurrentLineWhileRunning Then If Options.InterpreterDelay Then Common.Wait Options.InterpreterDelay
    End If
    
    If Program.State And psfPaused Then
      Form1.CodeEditor.SetArrow CurrentLine
      Common.Pause
    End If
    If bCancel Then Exit For
    If GetInputState Then DoEvents
  Next
End With
End Sub

Private Function ExecCommand(C As Command, CurrentProcedure As Long) As Variant
Dim i As Long


Select Case C.Index
Case cmdFD
  Turtle.MoveForward GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdRT
  Turtle.TurnRight GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdLT
  Turtle.TurnRight -GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdBK
  Turtle.MoveForward -GetParameterValue(C.Parameters(0), CurrentProcedure)
  
Case cmdMath
  ExecCommand = CalcExpression(C.Parameters, CurrentProcedure)
  
Case cmdRepeat
  For i = 1 To GetParameterValue(C.Parameters(0), CurrentProcedure)
    Call ExecuteBlock(CurrentProcedure, C.Parameters(1).ValIndex)
    If bCancel Then Exit For
  Next
  If bCancel Then If CancelReason = crLoopExit Then bCancel = False: CancelReason = 0
  
Case cmdWhile
  Do While GetParameterValue(C.Parameters(0), CurrentProcedure)
    Call ExecuteBlock(CurrentProcedure, C.Parameters(1).ValIndex)
    If bCancel Then Exit Do
  Loop
  If bCancel Then If CancelReason = crLoopExit Then bCancel = False: CancelReason = 0
  
Case cmdUntil
  Do Until GetParameterValue(C.Parameters(0), CurrentProcedure)
    Call ExecuteBlock(CurrentProcedure, C.Parameters(1).ValIndex)
    If bCancel Then Exit Do
  Loop
  If bCancel Then If CancelReason = crLoopExit Then bCancel = False: CancelReason = 0
  
Case cmdIf
  If GetParameterValue(C.Parameters(0), CurrentProcedure) Then
    Call ExecuteBlock(CurrentProcedure, C.Parameters(1).ValIndex)
  Else
    i = C.Parameters(2).ValIndex
    If i <> -1 Then Call ExecuteBlock(CurrentProcedure, i)
  End If
  
Case cmdCall
  ExecCommand = GetParameterValue(C.Parameters(0), CurrentProcedure)
  
Case cmdSetTurtlePos
  Turtle.SetPos GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure)
Case cmdSetTurtleX
  Turtle.SetX GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdSetTurtleY
  Turtle.SetY GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdSetTurtleAngle
  Turtle.SetAngle GetParameterValue(C.Parameters(0), CurrentProcedure)
  
Case cmdGetTurtleX
  ExecCommand = Turtle.GetX
Case cmdGetTurtleY
  ExecCommand = Turtle.GetY
Case cmdGetTurtleAngle
  ExecCommand = Turtle.GetAngle

Case cmdGetPointColor
  i = Turtle.GetCurrentPointColor()
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, i And &HFF&)
  Call SetVariableValue(C.Parameters(1).ValIndex, CurrentProcedure, (i And &HFF00&) \ &H100&)
  Call SetVariableValue(C.Parameters(2).ValIndex, CurrentProcedure, (i And &HFF0000) \ &H10000)
  
Case cmdSetAlpha
  Turtle.SetAlpha GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdWait
  Common.Wait GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdPU
  Turtle.SetPen False
Case cmdPD
  Turtle.SetPen True
Case cmdMake
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdAdd
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, GetParameterValue(C.Parameters(0), CurrentProcedure) + GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdSub
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, GetParameterValue(C.Parameters(0), CurrentProcedure) - GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdMul
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, GetParameterValue(C.Parameters(0), CurrentProcedure) * GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdDiv
  Call SetVariableValue(C.Parameters(0).ValIndex, CurrentProcedure, GetParameterValue(C.Parameters(0), CurrentProcedure) / GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdRnd
  ExecCommand = CSng(Rnd * GetParameterValue(C.Parameters(0), CurrentProcedure))
Case cmdSin
  ExecCommand = CSng(Sin(GetParameterValue(C.Parameters(0), CurrentProcedure) * Deg2Rad))
Case cmdCos
  ExecCommand = CSng(Cos(GetParameterValue(C.Parameters(0), CurrentProcedure) * Deg2Rad))
Case cmdTan
  ExecCommand = CSng(Tan(GetParameterValue(C.Parameters(0), CurrentProcedure) * Deg2Rad))
Case cmdATan
  ExecCommand = CSng(Atn(GetParameterValue(C.Parameters(0), CurrentProcedure)) * Rad2Deg)
Case cmdASin
  ExecCommand = CSng(ASin(GetParameterValue(C.Parameters(0), CurrentProcedure)) * Rad2Deg)
Case cmdACos
  ExecCommand = CSng(ACos(GetParameterValue(C.Parameters(0), CurrentProcedure)) * Rad2Deg)
  
Case cmdSqr
  ExecCommand = CSng(GetParameterValue(C.Parameters(0), CurrentProcedure) ^ (1 / GetParameterValue(C.Parameters(1), CurrentProcedure)))
Case cmdPower
  ExecCommand = CSng(GetParameterValue(C.Parameters(0), CurrentProcedure) ^ GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdLog
  ExecCommand = CSng(Log(GetParameterValue(C.Parameters(0), CurrentProcedure)) / Log(GetParameterValue(C.Parameters(1), CurrentProcedure)))
Case cmdLn
  ExecCommand = CSng(Log(GetParameterValue(C.Parameters(0), CurrentProcedure)))
Case cmdExp
  ExecCommand = CSng(Exp(GetParameterValue(C.Parameters(0), CurrentProcedure)))
  
Case cmdRound
  ExecCommand = CSng(Round(GetParameterValue(C.Parameters(0), CurrentProcedure), CLng(GetParameterValue(C.Parameters(1), CurrentProcedure))))
Case cmdFix
  ExecCommand = Fix(GetParameterValue(C.Parameters(0), CurrentProcedure))
  
Case cmdPC
  Turtle.ChangePenColor QBColor(GetParameterValue(C.Parameters(0), CurrentProcedure))
Case cmdPCRGB
  Turtle.ChangePenColor RGBJoin(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure), GetParameterValue(C.Parameters(2), CurrentProcedure))
Case cmdBG
  Turtle.ChangeBackgroundColor QBColor(GetParameterValue(C.Parameters(0), CurrentProcedure))
Case cmdBGRGB
  Turtle.ChangeBackgroundColor RGBJoin(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure), GetParameterValue(C.Parameters(2), CurrentProcedure))
Case cmdHT
  Turtle.SetTurtleVisibility False
Case cmdST
  Turtle.SetTurtleVisibility True
Case cmdSetDrawWidth
  Turtle.SetDrawWidth GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdFloodFill
  Turtle.FloodFill
Case cmdClear
  Turtle.Clear
Case cmdHome
  Turtle.MoveHome
Case cmdDraw
  Turtle.MoveHome
  Turtle.Clear
Case cmdEnableBackBuffer
  Turtle.SetBackBuffer True
Case cmdDisableBackBuffer
  Turtle.SetBackBuffer False
Case cmdEnableAutoUpdate
  Turtle.SetAutoUpdate True
Case cmdDisableAutoUpdate
  Turtle.SetAutoUpdate False
Case cmdUpdate
  Turtle.Update
  
Case cmdMoveDelay
  Turtle.SetMoveDelay GetParameterValue(C.Parameters(0), CurrentProcedure)
  
Case cmdDrawText
  Turtle.DrawText GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdSetFont
  Turtle.SetFont GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure), GetParameterValue(C.Parameters(2), CurrentProcedure)
Case cmdPenMode
  Turtle.SetPenMode GetParameterValue(C.Parameters(0), CurrentProcedure)
  
Case cmdKeyDown
  ExecCommand = -CLng((GetAsyncKeyState(GetParameterValue(C.Parameters(0), CurrentProcedure)) And &HFF&) <> 0)
  
Case cmdInkey
  ExecCommand = Form1.KeyboardStack.Pop

Case cmdStrLen
  ExecCommand = Len(CStr(GetParameterValue(C.Parameters(0), CurrentProcedure)))
Case cmdStrLeft
  ExecCommand = Left(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdStrRight
  ExecCommand = Right(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure))
Case cmdStrMid
  If C.NumParameters = 3 Then
    ExecCommand = Mid(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure), GetParameterValue(C.Parameters(2), CurrentProcedure))
  Else
    ExecCommand = Mid(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure))
  End If
  
Case cmdAsc
  ExecCommand = Asc(GetParameterValue(C.Parameters(0), CurrentProcedure))
Case cmdChr
  ExecCommand = Chr(GetParameterValue(C.Parameters(0), CurrentProcedure))
Case cmdString
  ExecCommand = String(GetParameterValue(C.Parameters(0), CurrentProcedure), GetParameterValue(C.Parameters(1), CurrentProcedure))
  
Case cmdPrintConsole
  Form1.Console.PrintText GetParameterValue(C.Parameters(0), CurrentProcedure), &H808080, ciLog
Case cmdReturn
  ActiveProcedures(NumActiveProcedures - 1).ReturnValue = GetParameterValue(C.Parameters(0), CurrentProcedure)
Case cmdExitLoop
  Program.bCancel = True
  Program.CancelReason = crLoopExit
Case cmdExitProcedure
  Program.bCancel = True
  Program.CancelReason = crProcedureExit
Case cmdTerminate
  Program.bCancel = True
  Program.CancelReason = crProgramExit
End Select
End Function

Private Function GetParameterValue(P As Parameter, CurrentProcedure As Long) As Variant
If P.Type = ptConstant Then
  GetParameterValue = P.Value
ElseIf P.Type = ptVariable Then
  If (P.ValIndex And vfComplexAddress) = 0 Then
    If (P.ValIndex And vfPublic) = 0 Then
      GetParameterValue = ActiveProcedures(NumActiveProcedures - 1).LocalVariables(P.ValIndex)
    Else
      GetParameterValue = GlobalVariables(P.ValIndex Xor vfPublic).Value
    End If
  Else
    GetParameterValue = GetComplexVariableValue(P.ValIndex Xor vfComplexAddress, CurrentProcedure)
  End If
ElseIf P.Type = ptNativeCommand Then
  GetParameterValue = ExecCommand(Program.Commands(P.ValIndex), CurrentProcedure)
ElseIf P.Type = ptProcedure Then
  GetParameterValue = ExecuteProcedure(Program.Commands(P.ValIndex), CurrentProcedure)
End If
End Function

Private Function GetComplexVariableValue(ComplexAddressArrayIndex As Long, Procedure As Long) As Variant

With Program.ComplexAddressArrays(ComplexAddressArrayIndex And Not vfPublic)
  If ComplexAddressArrayIndex And vfPublic Then 'Global var
    GetComplexVariableValue = GetComplexVariableValueEx(Program.GlobalVariables(.BaseVariable).Value, _
                                                        Program.GlobalVariables(.BaseVariable).VarType, _
                                                        ComplexAddressArrayIndex Xor vfPublic, _
                                                        0)
  Else
    GetComplexVariableValue = GetComplexVariableValueEx(ActiveProcedures(NumActiveProcedures - 1).LocalVariables(.BaseVariable), _
                                                        Program.Procedures(Procedure).LocalVariablesInfo(.BaseVariable).VarType, _
                                                        ComplexAddressArrayIndex, _
                                                        0)
  End If

End With
End Function

Private Function GetComplexVariableValueEx(Value As Variant, VariableType As Long, ComplexAddressArrayIndex As Long, ComplexAddressArrayPosition As Long) As Variant
With Program.ComplexAddressArrays(ComplexAddressArrayIndex)
  If (VarType(Value) And vbArray) = 0 Then
    Value = Common.CreateVariantArray(Program.Structs(VariableType).NumMembers)
  End If
  
  If ComplexAddressArrayPosition = .NumLevels - 1 Then
    GetComplexVariableValueEx = Value(.Addresses(ComplexAddressArrayPosition))
  Else
    GetComplexVariableValueEx = GetComplexVariableValueEx(Value(.Addresses(ComplexAddressArrayPosition)), _
                                                          Program.Structs(VariableType).Members(.Addresses(ComplexAddressArrayPosition)).VarType, _
                                                          ComplexAddressArrayIndex, _
                                                          ComplexAddressArrayPosition + 1)
  End If
End With
End Function

Private Sub SetVariableValue(VarIndex As Long, CurrentProcedure As Long, Value As Variant)
If (VarIndex And vfComplexAddress) = 0 Then
  If (VarIndex And vfPublic) = 0 Then
    ActiveProcedures(NumActiveProcedures - 1).LocalVariables(VarIndex) = Value
  Else
    GlobalVariables(VarIndex Xor vfPublic).Value = Value
  End If
Else
  Call SetComplexVariableValue(VarIndex Xor vfComplexAddress, CurrentProcedure, Value)
End If
End Sub

Private Sub SetComplexVariableValue(ComplexAddressArrayIndex As Long, Procedure As Long, NewValue As Variant)
With Program.ComplexAddressArrays(ComplexAddressArrayIndex And Not vfPublic)

  If ComplexAddressArrayIndex And vfPublic Then 'Global var
    Call SetComplexVariableValueEx(Program.GlobalVariables(.BaseVariable).Value, _
                                   Program.GlobalVariables(.BaseVariable).VarType, _
                                   ComplexAddressArrayIndex Xor vfPublic, _
                                   0, _
                                   NewValue)
  Else
    Call SetComplexVariableValueEx(ActiveProcedures(NumActiveProcedures - 1).LocalVariables(.BaseVariable), _
                                   Program.Procedures(Procedure).LocalVariablesInfo(.BaseVariable).VarType, _
                                   ComplexAddressArrayIndex, _
                                   0, _
                                   NewValue)
  End If

End With
End Sub

Private Sub SetComplexVariableValueEx(Value As Variant, VariableType As Long, ComplexAddressArrayIndex As Long, ComplexAddressArrayPosition As Long, NewValue As Variant)
With Program.ComplexAddressArrays(ComplexAddressArrayIndex)
  If (VarType(Value) And vbArray) = 0 Then
    Value = Common.CreateVariantArray(Program.Structs(VariableType).NumMembers)
  End If
  
  If ComplexAddressArrayPosition = .NumLevels - 1 Then
    Value(.Addresses(ComplexAddressArrayPosition)) = NewValue
  Else
    Call SetComplexVariableValueEx(Value(.Addresses(ComplexAddressArrayPosition)), _
                                   Program.Structs(VariableType).Members(.Addresses(ComplexAddressArrayPosition)).VarType, _
                                   ComplexAddressArrayIndex, _
                                   ComplexAddressArrayPosition + 1, _
                                   NewValue)
  End If
End With
End Sub

Private Function CalcExpression(P() As Parameter, CurrentProcedure As Long) As Variant
Select Case P(1).ValIndex
  Case [eoA+B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) + GetParameterValue(P(2), CurrentProcedure)
  Case [eoA-B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) - GetParameterValue(P(2), CurrentProcedure)
  Case [eoA*B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) * GetParameterValue(P(2), CurrentProcedure)
  Case [eoA/B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) / GetParameterValue(P(2), CurrentProcedure)
  Case [eoA^B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) ^ GetParameterValue(P(2), CurrentProcedure)
  Case [eoA=B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) = GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA<>B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) <> GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA<B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) < GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA>B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) > GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA=>B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) >= GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA<=B]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure) <= GetParameterValue(P(2), CurrentProcedure)))
  Case [eoAAndB]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure)) And CBool(GetParameterValue(P(2), CurrentProcedure)))
  Case [eoAOrB]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure)) Or CBool(GetParameterValue(P(2), CurrentProcedure)))
  Case [eoAXorB]: CalcExpression = -CLng(CBool(GetParameterValue(P(0), CurrentProcedure)) Xor CBool(GetParameterValue(P(2), CurrentProcedure)))
  Case [eoA&B]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) & GetParameterValue(P(2), CurrentProcedure)
  Case [eoAModB]: CalcExpression = GetParameterValue(P(0), CurrentProcedure) Mod GetParameterValue(P(2), CurrentProcedure)
  Case [eoAModfB]: CalcExpression = Modf(GetParameterValue(P(0), CurrentProcedure), GetParameterValue(P(2), CurrentProcedure))
End Select
End Function


Private Function GetStartParams(Params() As Parameter) As Boolean
Dim S As String
Dim W As Long
Dim P() As String
S = InputBox("Enter start parameter(s)", "Start Parameters")
If StrPtr(S) Then
  P = Split(S, " ")
  If UBound(P) + 1 = Procedures(MainProcedure).NumParameters Then
    ReDim Params(0 To UBound(P))
    For W = 0 To UBound(P)
      With Params(W)
        .Type = ptConstant
        If IsNumeric(P(W)) Then
          .Value = CSng(P(W))
        Else
          .Value = P(W)
        End If
      End With
    Next
    GetStartParams = True
  Else
    Err.Raise 10013, , "Expected: " & Procedures(MainProcedure).NumParameters & " parameter(s)"
  End If
End If
End Function

Private Function RGBJoin(R As Long, G As Long, B As Long)
If R > 255 Then R = 255
If G > 255 Then G = 255
If B > 255 Then B = 255
RGBJoin = R Or (G * &H100&) Or (B * &H10000)
End Function

Private Sub ClearKeyStack()
Dim W As Long
For W = 0 To 255
  Do While GetAsyncKeyState(W) And &HFF: Loop
Next
End Sub

