Attribute VB_Name = "LineAnalyzer"
Option Explicit

Public Type CommandPositions
  CommandStart As Long
  CommandEnd As Long
  CommandLength As Long
  ParametersStart As Long
  ParametersEnd As Long
  ParametersLength As Long
End Type

Private Type StringParameter
  Pos As Long
  Param As String
End Type

Private Type ResolvedCommand
  Command As String
  NumParams As Long
  Params() As StringParameter
End Type

Public Type CommandParserResult
  Command As Command
  CommandArrayIndex As Long
End Type

Public Function RemoveRedundantSpaces(Line As String) As String
Dim W As Long
Dim bString As Boolean
Dim Char As String * 1
Dim LastChar As String * 1
Dim NextChar As String * 1
Dim result As String
Dim Eat As Boolean

For W = 1 To Len(Line)
  Char = Mid$(Line, W, 1)
  If W <> Len(Line) Then NextChar = Mid$(Line, W + 1, 1)
  
  If Char = """" Then bString = Not bString
  Eat = False
  
  If bString = False Then
    If Char = " " Then
      If NextChar = " " Then
        Eat = True
      ElseIf NextChar = ")" Then
        Eat = True
      ElseIf LastChar = "(" Then
        Eat = True
      End If
    End If
  End If
  
  If Eat = False Then result = result & Char
  LastChar = Char
Next
RemoveRedundantSpaces = result
End Function

Public Function ParseCommand(Line As String, StartPos As Long, TextCodeLine As Long, CurrentProcedure As Long, CurrentBlock As Long) As CommandParserResult
Dim PrimaryCommand As Boolean
Dim Positions As CommandPositions
Dim ResCmd As ResolvedCommand
Dim IdCmd As Parameter
Dim DBEntry As CommandParams
Dim Command As Command
Dim ParserResult As CommandParserResult
Dim W As Long

PrimaryCommand = StartPos = 1

Positions = GetCommandPositions(Line, StartPos)
ResCmd = ResolveCommand(Line, Positions)

IdCmd = IdentifySegment(ResCmd.Command, CurrentProcedure)

Command.Type = IdCmd.Type
Command.Index = IdCmd.ValIndex

If IdCmd.Type = ptNativeCommand Then
  DBEntry = CommandsDB.CommandParams(Command.Index)
ElseIf IdCmd.Type = ptProcedure Then
  DBEntry = CommandsDB.GetProcedureInfo(Command.Index)
ElseIf IdCmd.Type = ptVariable Then
  DBEntry = CommandsDB.CommandParams(cmdMake)
  DBEntry.MaxParameterCount = 2
  DBEntry.MinParameterCount = 2
  ReDim Preserve DBEntry.PrametersInfo(0 To 1)
  DBEntry.PrametersInfo(0).AllowedTytes = ptOperand
  DBEntry.PrametersInfo(1).AllowedTytes = 0
Else
  Err.Raise 10003, , "Expected: Command or Procedure"
End If

If Not DBEntry.ParserLevel1Only Then
  
  If ResCmd.NumParams < DBEntry.MinParameterCount Then Err.Raise 10002, , "Expected: " & DBEntry.MinParameterCount & " parameter(s)"
  If ResCmd.NumParams > DBEntry.MaxParameterCount Then Err.Raise 10002, , "Expected: " & DBEntry.MaxParameterCount & " parameter(s)"
  If ((DBEntry.Usage And cuPrimaryCommand) = 0) And PrimaryCommand Then Err.Raise 10004, , "Command can only be used as a Function"
  If ((DBEntry.Usage And cuParameter) = 0) And (PrimaryCommand = False) Then Err.Raise 10005, , "Command can not be used as a Function"
  
  For W = 0 To ResCmd.NumParams - 1
    Command.NumParameters = Command.NumParameters + 1
    ReDim Preserve Command.Parameters(0 To Command.NumParameters - 1)
    
    If IsExpressionFunction(ResCmd.Params(W).Param) Then
      ParserResult = ParseCommand(Line, ResCmd.Params(W).Pos, TextCodeLine, CurrentProcedure, CurrentBlock)
      Command.Parameters(Command.NumParameters - 1).Type = ParserResult.Command.Type
      Command.Parameters(Command.NumParameters - 1).ValIndex = ParserResult.CommandArrayIndex
    Else
      Command.Parameters(Command.NumParameters - 1) = IdentifySegment(ResCmd.Params(W).Param, CurrentProcedure)
    End If
  Next
  
  
  If ResCmd.NumParams Then
    For W = 0 To Min(ResCmd.NumParams - 1, UBound(DBEntry.PrametersInfo))
      If DBEntry.PrametersInfo(W).AllowedTytes Then
        If (DBEntry.PrametersInfo(W).AllowedTytes And Command.Parameters(W).Type) Then
        Else
          Err.Raise 10004, , "Expected: " & "Parameter " & W & " to be " & CommandsDB.ParameterTypeAsString(DBEntry.PrametersInfo(W).AllowedTytes)
        End If
      End If
      
      If Command.Parameters(W).Type = ptVariable Then
        Dim A As Long, B As Long
        A = DBEntry.PrametersInfo(W).VariableType
        B = GetVariableInfo(Command.Parameters(W).ValIndex, CurrentProcedure).VarType
        If A <> B Then
          Err.Raise 10000, , "Types incompatible"
        End If
      End If
    Next
  End If
  
  If IdCmd.Type = ptVariable Then
    If Command.Parameters(0).Type = ptOperand And Command.Parameters(0).ValIndex = [eoA=B] Then
      Command.Type = ptNativeCommand
      Command.Index = cmdMake
      Command.Parameters(0).Type = ptVariable
      Command.Parameters(0).ValIndex = IdCmd.ValIndex
    Else
      Err.Raise 10000, , "Expected: '='"
    End If
  End If
  
  
  ParseCommand.Command = Command
  
  If DBEntry.DoNotExecute = False Then
    If PrimaryCommand Then
      ParseCommand.CommandArrayIndex = Program.AddCommand(Command, TextCodeLine, CurrentProcedure, CurrentBlock)
    Else
      ParseCommand.CommandArrayIndex = Program.AddCommand(Command, TextCodeLine)
    End If
  Else
    ParseCommand.CommandArrayIndex = -1
  End If
Else
  ParseCommand.Command.Index = -1
  ParseCommand.CommandArrayIndex = -1
End If
  
End Function

Private Function ResolveCommand(Line As String, Positions As CommandPositions) As ResolvedCommand
Dim W As Long
Dim Char As String
Dim bString As Boolean
Dim LastParamStart As Long

With ResolveCommand
  .Command = Mid$(Line, Positions.CommandStart, Positions.CommandLength)
  
  If Positions.ParametersLength Then
    LastParamStart = Positions.ParametersStart
    For W = Positions.ParametersStart To Positions.ParametersEnd
      Char = Mid$(Line, W, 1)

      If Char = " " Then
        If bString = False Then
          .NumParams = .NumParams + 1
          ReDim Preserve .Params(0 To .NumParams - 1)
          .Params(.NumParams - 1).Pos = LastParamStart
          .Params(.NumParams - 1).Param = Mid$(Line, LastParamStart, W - 1 - LastParamStart + 1)
          
          LastParamStart = W + 1
        End If
      ElseIf Char = "(" Then
        W = FindClosingBracket(Line, W)
      ElseIf Char = """" Then
        bString = Not bString
      End If
    Next
    
    .NumParams = .NumParams + 1
    ReDim Preserve .Params(0 To .NumParams - 1)
    .Params(.NumParams - 1).Pos = LastParamStart
    .Params(.NumParams - 1).Param = Mid$(Line, LastParamStart, W - 1 - LastParamStart + 1)
  End If
End With
End Function

Private Function GetCommandPositions(Line As String, StartPos As Long) As CommandPositions
Dim PrimaryCommand As Boolean
Dim SpacePos As Long
Dim BracketPos As Long
Dim HasParams As Boolean
PrimaryCommand = StartPos = 1

With GetCommandPositions
  .CommandStart = StartPos
  
  If PrimaryCommand Then
    .CommandEnd = InStr(StartPos, Line, " ") - 1
    
    If .CommandEnd = -1 Then
      .CommandEnd = Len(Line)
    Else
      HasParams = True
    End If
  Else
    BracketPos = InStr(StartPos, Line, "(")
    SpacePos = InStr(StartPos, Line, " ")
    
    If BracketPos = 0 And SpacePos = 0 Then
      .CommandEnd = Len(Line)
    Else
      If (SpacePos < BracketPos And SpacePos <> 0) Or BracketPos = 0 Then
        .CommandEnd = SpacePos - 1
      Else
        .CommandEnd = BracketPos - 1
        HasParams = True
      End If
    End If
  End If
  .CommandLength = .CommandEnd - .CommandStart + 1
  If HasParams Then
    
    .ParametersStart = .CommandEnd + 2
    If PrimaryCommand Then
      .ParametersEnd = Len(Line)
    Else
      .ParametersEnd = FindClosingBracket(Line, .ParametersStart - 1) - 1
    End If
    .ParametersLength = .ParametersEnd - .ParametersStart + 1
    If .ParametersLength = 0 Then
      .ParametersStart = 0
      .ParametersEnd = 0
    End If
  End If
End With
End Function

Private Function FindClosingBracket(Line As String, OpeningBracket As Long)
Dim BracketsOpen As Long
Dim Char As String
Dim W As Long
For W = OpeningBracket + 1 To Len(Line)
  Char = Mid$(Line, W, 1)
  If Char = "(" Then
    BracketsOpen = BracketsOpen + 1
  ElseIf Char = ")" Then
    BracketsOpen = BracketsOpen - 1
    If BracketsOpen < 0 Then
      FindClosingBracket = W
      Exit For
    End If
  End If
Next

If W > Len(Line) Then Err.Raise 10000, , "Missing closeing Bracket"
End Function

Private Function IsExpressionFunction(Expression As String) As Boolean
If Left$(Expression, 1) = """" Then
  IsExpressionFunction = False
ElseIf InStr(1, Expression, "(") Then
  IsExpressionFunction = True
ElseIf IsNumeric(Expression) Then
  IsExpressionFunction = False
Else
  If GetCommandIndex(Expression) Then
    IsExpressionFunction = True
  Else
    IsExpressionFunction = FindProcedure(Expression) <> -1
  End If
End If

End Function

Private Function IdentifySegment(Param As String, CurrentProcedure As Long) As Parameter
Dim W As Long
Dim UDTSep As Long
Dim S As String

If Len(Param) = 0 Then
  IdentifySegment.Type = ptNativeCommand '!!!!!
  IdentifySegment.ValIndex = cmdMath
ElseIf IsNumeric(Param) Then 'Number
  IdentifySegment.Type = ptConstant
  IdentifySegment.Value = CSng(Val(Param))
ElseIf Left$(Param, 1) = """" And Right$(Param, 1) = """" Then 'String
  IdentifySegment.Type = ptConstant
  IdentifySegment.Value = Mid$(Param, 2, Len(Param) - 2)
Else 'Variable, Constant or Function
  UDTSep = InStr(1, Param, ".") 'UDT Zugriff
  If UDTSep Then
    S = LCase$(Left$(Param, UDTSep - 1))
  Else
    S = LCase$(Param)
  End If
  
  
  'Command
  IdentifySegment.ValIndex = GetCommandIndex(S)
  If IdentifySegment.ValIndex Then
    IdentifySegment.Type = ptNativeCommand
    Exit Function
  End If
  
  'Lokale Konstanten
  If CurrentProcedure <> -1 Then
    With Procedures(CurrentProcedure)
      For W = 0 To .NumLocalConstants - 1
        If LCase$(.LocalConstants(W).Name) = S Then
          IdentifySegment.Type = ptConstant
          IdentifySegment.Value = .LocalConstants(W).Value
          Exit Function
        End If
      Next
    End With
  End If
  
  'Globale Konstanten
  For W = 0 To NumGlobalConstants - 1
    If LCase$(GlobalConstants(W).Name) = S Then
      IdentifySegment.Type = ptConstant
      IdentifySegment.Value = GlobalConstants(W).Value
      Exit Function
    End If
  Next
  
  'Lokale Variablen
  If CurrentProcedure <> -1 Then
    With Procedures(CurrentProcedure)
      For W = 0 To .NumLocalVariables - 1
        If LCase$(.LocalVariablesInfo(W).Name) = S Then
          IdentifySegment.Type = ptVariable
          
          If UDTSep Then
            IdentifySegment.ValIndex = ParseUDTAccess(CurrentProcedure, W, Mid$(Param, UDTSep + 1)) Or vfComplexAddress
          Else
            IdentifySegment.ValIndex = W
          End If
          Exit Function
        End If
      Next
    End With
  End If
  
  'Prozedure
  IdentifySegment.ValIndex = FindProcedure(S)
  If IdentifySegment.ValIndex <> -1 Then
    IdentifySegment.Type = ptProcedure
    Exit Function
  End If
  
  'Globale Variable
  For W = 0 To NumGlobalVariables - 1
    If LCase$(GlobalVariables(W).Name) = S Then
      IdentifySegment.Type = ptVariable
      
      If UDTSep Then
        IdentifySegment.ValIndex = ParseUDTAccess(-1, W, Mid$(Param, UDTSep + 1)) Or vfComplexAddress Or vfPublic
      Else
        IdentifySegment.ValIndex = W Or vfPublic
      End If
      Exit Function
    End If
  Next
  
  'Operand
  IdentifySegment.ValIndex = GetOperandIndex(S)
  If IdentifySegment.ValIndex Then
    IdentifySegment.Type = ptOperand
    Exit Function
  End If
    
  Err.Raise 1005, , "'" & Param & "': Expression not defined"
End If
End Function

Private Function ParseUDTAccess(Procedure As Long, BaseVarIndex As Long, AccessString As String) As Long
Dim Seg() As String
Dim W As Long
Dim VarType As Long
Dim Adr As Long
Dim Adresses() As Long

Seg = Split(AccessString, ".")

If Procedure = -1 Then
  VarType = Program.GlobalVariables(BaseVarIndex).VarType
Else
  VarType = Program.Procedures(Procedure).LocalVariablesInfo(BaseVarIndex).VarType
End If

ReDim Adresses(0 To UBound(Seg))

For W = 0 To UBound(Seg)
  Adr = FindUDTMember(VarType, Seg(W))
  If Adr <> -1 Then
    Adresses(W) = Adr
  Else
    Err.Raise 10000, , "'" & Seg(W) & "' is not a member of '" & Program.Structs(VarType).Name & "'"
  End If
  VarType = Program.Structs(VarType).Members(Adr).VarType
Next

ParseUDTAccess = Program.AddComplexAddressArray(BaseVarIndex, UBound(Adresses) + 1, Adresses)
End Function

Private Function FindUDTMember(VarTypeIndex As Long, MemberName As String) As Long
Dim W As Long
FindUDTMember = -1
With Program.Structs(VarTypeIndex)
  For W = 0 To .NumMembers - 1
    If LCase$(.Members(W).Name) = LCase$(MemberName) Then
      FindUDTMember = W
      Exit Function
    End If
  Next
End With
End Function

Public Function FindProcedure(ProcName As String) As Long
Dim W As Long
FindProcedure = -1
For W = 0 To NumProcedures - 1
  If LCase$(Procedures(W).Name) = LCase$(ProcName) Then
    FindProcedure = W
    Exit For
  End If
Next
End Function

Public Function GetCommandIndex(ByVal Cmd As String) As enmCommand
Dim W As Long
Dim N As Long
Cmd = LCase$(Cmd)
For W = 1 To CommandsDB.NumNativeCommands
  With CommandsDB.CommandParams(W)
    For N = 0 To .NumNames - 1
      If LCase$(.Names(N)) = Cmd Then
        GetCommandIndex = W
      End If
    Next
  End With
Next

End Function

Public Function GetOperandIndex(Op As String) As enmExpressionOperand
If Len(Op) <= 4 Then
  Select Case LCase$(Op)
    Case "+":  GetOperandIndex = [eoA+B]
    Case "-":  GetOperandIndex = [eoA-B]
    Case "*":  GetOperandIndex = [eoA*B]
    Case "/":  GetOperandIndex = [eoA/B]
    Case "^":  GetOperandIndex = [eoA^B]
    Case "=":  GetOperandIndex = [eoA=B]
    Case "<>", "><":  GetOperandIndex = [eoA<>B]
    Case "<":  GetOperandIndex = [eoA<B]
    Case ">":  GetOperandIndex = [eoA>B]
    Case "=>", ">=":  GetOperandIndex = [eoA=>B]
    Case "<=", "=<":  GetOperandIndex = [eoA<=B]
    Case "and": GetOperandIndex = eoAAndB
    Case "or": GetOperandIndex = eoAOrB
    Case "xor": GetOperandIndex = eoAXorB
    Case "&":  GetOperandIndex = [eoA&B]
    Case "mod":  GetOperandIndex = [eoAModB]
    Case "modf": GetOperandIndex = [eoAModfB]
  End Select
End If
End Function


Public Function GetVariableInfo(VarIndex As Long, CurrentProcedure As Long) As VariableInfo
Dim i As VariableInfo
Dim W As Long

If (VarIndex And vfComplexAddress) = 0 Then
  If (VarIndex And vfPublic) Then
    With Program.GlobalVariables(VarIndex Xor vfPublic)
      GetVariableInfo.VarType = .VarType
      GetVariableInfo.Name = .Name
    End With
  Else
    GetVariableInfo = Program.Procedures(CurrentProcedure).LocalVariablesInfo(VarIndex)
  End If
Else
  With Program.ComplexAddressArrays(VarIndex And vfVarIndex)
    i = GetVariableInfo(.BaseVariable, CurrentProcedure)
    For W = 0 To .NumLevels - 1
      i = Program.Structs(i.VarType).Members(.Addresses(W))
    Next
    GetVariableInfo = i
  End With
End If
End Function

