Attribute VB_Name = "Parser"
Option Explicit

Private Const TypeSepChar As String = "!"

Private CurrentLine As Long

Public Function Parse(Code() As String) As Boolean
On Error GoTo Err

Program.Clear

Program.Code = Code

Constants.Init

Program.State = psParsing1
Form1.ProgramStateChanged
If Options.ClearConsoleBeforeRun Then Form1.Console.Clear Else If Form1.Console.IsEmpty = False Then Form1.Console.AddEmptyLine
Form1.Console.PrintText "Parsing Code (1)", 0, ciParse

Call PreProcessing(-1, 0)
If Program.bCancel Then
  Program.State = psInactive
  Form1.ProgramStateChanged
  If Options.DebugMode Then Form1.ProcedureInfoChanged ""
  Exit Function
End If

If MainProcedure = -1 Then
  CurrentLine = -1
  Err.Raise 10004, "", "Procedure Main not found"
End If

Form1.Console.PrintText "Found " & Program.NumProcedures & " procedure(s)", &H8000&, ciParseOK

Program.State = psParsing2
Form1.ProgramStateChanged
Form1.Console.PrintText "Parsing Code (2)", 0, ciParse

Dim W As Long
For W = 0 To NumProcedures - 1
  Call MainProcessing(W, 0, Procedures(W).StartLine)
Next
If Program.bCancel Then
  Program.State = psInactive
  Form1.ProgramStateChanged
  If Options.DebugMode Then Form1.ProcedureInfoChanged ""
  Exit Function
End If

If Options.DebugMode Then Form1.ProcedureInfoChanged ""

Form1.Console.PrintText "Found " & Program.NumCommands & " command(s)", &H8000&, ciParseOK

Parse = True
Exit Function
Err:
If CurrentLine <> -1 Then
  Form1.CodeEditor.SetArrow CurrentLine
  Form1.Console.PrintText "Error in line " & CurrentLine, vbRed, ciError
End If
Form1.Console.PrintText Err.Description, vbRed, ciError
Program.State = psInactive
Form1.ProgramStateChanged
If Options.DebugMode Then Form1.ProcedureInfoChanged ""

If CurrentLine <> -1 Then Form1.CodeEditor.SelectLine CurrentLine
End Function

Private Sub PreProcessing(CurrentProcedure As Long, StartLine As Long)
Dim UB As Long
Dim CommentPos As Long
Dim SpacePos As Long
Dim Line As String
Dim NewProcedureIndex As Long
Dim Command As enmCommand
Dim CommandParams() As String
Dim W As Long

CurrentLine = StartLine

If Options.ShowProcedureDetailsWhileParsing Then
  If CurrentProcedure <> -1 Then
    Form1.ProcedureInfoChanged Procedures(CurrentProcedure).Name
  Else
    Form1.ProcedureInfoChanged ""
  End If
  Form1.StatusBar.Refresh
End If

UB = UBound(Code)

Do Until CurrentLine > UB
  If Options.DebugMode Then If Options.ShowCurrentLineWhileParsing Then Form1.CodeEditor.SetArrow CurrentLine
  
  Line = Trim$(Code(CurrentLine))
  CommentPos = InStr(1, Line, "//")
  If CommentPos Then Line = RTrim$(Left$(Line, CommentPos - 1))
  Code(CurrentLine) = Line
  
  If Len(Line) Then
    SpacePos = InStr(1, Line, " ")
    If SpacePos Then
      Command = GetCommandIndex(Left$(Line, SpacePos - 1))
    Else
      Command = GetCommandIndex(Line)
    End If
    If Command Then
      If Command = cmdConst Then
        If SpacePos = 0 Then Err.Raise 10002, CurrentLine, "Expected: " & "Name"
        CommandParams = Split(Mid$(Line, SpacePos + 1), " ", 2)
        If UBound(CommandParams) <> 1 Then Err.Raise 10002, CurrentLine, "Expected: " & 2 & " Parameters"
        If IsValiedName(CommandParams(0)) = False Then Err.Raise 10012, , """" & CommandParams(0) & """ is no valid idetifier"
        
        If IsNumeric(CommandParams(1)) Then
          Program.AddConstant cgDefualt, CommandParams(0), CSng(Val(CommandParams(1))), CurrentProcedure
        ElseIf Left$(CommandParams(1), 1) = """" And Right$(CommandParams(1), 1) = """" Then
          Program.AddConstant cgDefualt, CommandParams(0), Mid$(CommandParams(1), 2, Len(CommandParams(1)) - 2), CurrentProcedure
        Else
          Err.Raise 10026, , CommandParams(0) & " is no vailid constant value"
        End If
        
      ElseIf Command = cmdDim Then
        If SpacePos = 0 Then Err.Raise 10002, CurrentLine, "Expected: " & "Name"
        CommandParams = Split(Mid$(Line, SpacePos + 1), " ")
        If IsValiedName(CommandParams(0)) = False Then Err.Raise 10012, , """" & CommandParams(0) & """ is no valid idetifier"
        
        Dim VI As VariableInfo
        For W = 0 To UBound(CommandParams)
          VI = ParseVariableDeclaration(CommandParams(W), CurrentProcedure)
          Program.AddVariable VI.Name, VI.VarType, CurrentProcedure, False
        Next
        
      ElseIf Command = cmdStruct Then
        If CurrentProcedure <> -1 Then Err.Raise 10013, , "'Struct' inside procedure not allowed"
        If SpacePos = 0 Then Err.Raise 10002, CurrentLine, "Expected: " & "Struct Name"
        CommandParams = Split(Mid$(Line, SpacePos + 1), " ")
        If IsValiedName(CommandParams(0)) = False Then Err.Raise 10012, , """" & CommandParams(0) & """ is no valid idetifier"
        
        Dim Members() As VariableInfo
        ReDim Members(0 To UBound(CommandParams) - 1)
        For W = 1 To UBound(CommandParams)
          Members(W - 1) = ParseVariableDeclaration(CommandParams(W), CurrentProcedure)
        Next
        Program.AddStruct CommandParams(0), UBound(Members) + 1, Members
        Erase Members
        
      ElseIf Command = cmdTo Then
        If CurrentProcedure <> -1 Then Err.Raise 10013, , "'To' inside procedure not allowed"
        If SpacePos = 0 Then Err.Raise 10002, CurrentLine, "Expected: " & "Procedure Name"
        CommandParams = Split(Mid$(Line, SpacePos + 1), " ")
        If IsValiedName(CommandParams(0)) = False Then Err.Raise 10012, , """" & CommandParams(0) & """ is no valid idetifier"
        
        NumProcedures = NumProcedures + 1
        ReDim Preserve Procedures(0 To NumProcedures - 1)
        NewProcedureIndex = NumProcedures - 1
        With Procedures(NewProcedureIndex)
          .Name = CommandParams(0)
          .StartLine = CurrentLine + 1
          ReDim .Blocks(0)
          .NumBlocks = 1
          .Blocks(0).Type = btNormalProcedure
          If UBound(CommandParams) > 0 Then 'Procedure has Parameters
            For W = 1 To UBound(CommandParams)
              If IsValiedName(CommandParams(W)) Then
                VI = ParseVariableDeclaration(CommandParams(W), CurrentProcedure)
                Call AddVariable(VI.Name, VI.VarType, NewProcedureIndex, True)
              Else
                Err.Raise 10012, , """" & CommandParams(W) & """ is no valid idetifier"
              End If
            Next
          End If
          If LCase$(.Name) = "main" Then
            If MainProcedure = -1 Then
              MainProcedure = NewProcedureIndex
            Else
              Err.Raise 10003, CurrentLine, "Only one Main Procedure allowed"
            End If
          End If
        End With
        If Options.DebugMode Then If Options.ShowCurrentLineWhileParsing Then If Options.ParserDelay Then Common.Wait Options.ParserDelay
        
        CurrentLine = CurrentLine + 1
        Call PreProcessing(NewProcedureIndex, CurrentLine)
      ElseIf Command = cmdEnd Then
        If CurrentProcedure <> -1 Then
          Procedures(CurrentProcedure).EndLine = CurrentLine - 1
          Exit Sub 'Sub Exit on Procedure End
        Else
          Err.Raise 10008, CurrentLine, "End outside procedure"
        End If
      Else
        If CurrentProcedure = -1 Then Err.Raise 10003, CurrentLine, "Code outside procedure not allowed"
      End If
    Else
      If CurrentProcedure = -1 Then Err.Raise 10003, CurrentLine, "Code outside procedure not allowed"
      'Err.Raise 10001, CurrentLine, "Unkonwn Command: " & Line
    End If
  End If
  
  If Options.DebugMode Then If Options.ShowCurrentLineWhileParsing Then If Options.ParserDelay Then Common.Wait Options.ParserDelay
  If Program.State And psfPaused Then
    Form1.CodeEditor.SetArrow CurrentLine
    Common.Pause
  End If
  
  If Program.bCancel Then Exit Do
  CurrentLine = CurrentLine + 1
Loop

If CurrentProcedure <> -1 Then CurrentLine = -1: Err.Raise 10014, , "Expected: 'End'"
End Sub

Private Sub MainProcessing(CurrentProcedure As Long, CurrentBlock As Long, StartLine As Long)
Dim Line As String
Dim ParserResult As CommandParserResult
Dim NewBlockIndex As Long

If Options.ShowProcedureDetailsWhileParsing Then Form1.ProcedureInfoChanged Procedures(CurrentProcedure).Name

For CurrentLine = StartLine To Procedures(CurrentProcedure).EndLine
  If Options.DebugMode Then If Options.ShowCurrentLineWhileParsing Then Form1.CodeEditor.SetArrow CurrentLine
  
  Line = Code(CurrentLine)
  If Len(Line) Then
    
    Line = RemoveRedundantSpaces(Line)
    ParserResult = LineAnalyzer.ParseCommand(Line, 1, CurrentLine, CurrentProcedure, CurrentBlock)
    With ParserResult
      
      If .Command.Index = cmdRepeat Or .Command.Index = cmdWhile Or .Command.Index = cmdUntil Then
        NewBlockIndex = AddBlock(btLoopBlock, CurrentProcedure)
        Call AddIndexParameter(.Command, NewBlockIndex)
        Call MainProcessing(CurrentProcedure, NewBlockIndex, CurrentLine + 1)
        
      ElseIf .Command.Index = cmdLoop Then
        If Program.Procedures(CurrentProcedure).Blocks(CurrentBlock).Type <> btLoopBlock Then Err.Raise 10003, , "Loop or EndRepeat without Repeat"
        Exit For
            
        
      ElseIf .Command.Index = cmdIf Then
        NewBlockIndex = AddBlock(btIfThenBlock, CurrentProcedure)
        Call AddIndexParameter(.Command, NewBlockIndex)
        Call MainProcessing(CurrentProcedure, NewBlockIndex, CurrentLine + 1)
        
        If LineAnalyzer.GetCommandIndex(Code(CurrentLine)) = cmdElse Then
          NewBlockIndex = AddBlock(btIfElseBlock, CurrentProcedure)
          Call AddIndexParameter(.Command, NewBlockIndex)
          Call MainProcessing(CurrentProcedure, NewBlockIndex, CurrentLine + 1)
        Else
          Call AddIndexParameter(.Command, -1)
        End If
        
        
      ElseIf .Command.Index = cmdElse Then
        If Program.Procedures(CurrentProcedure).Blocks(CurrentBlock).Type <> btIfThenBlock Then Err.Raise 10003, , "Else without If"
        Exit For
      ElseIf .Command.Index = cmdEndIf Then
        If Program.Procedures(CurrentProcedure).Blocks(CurrentBlock).Type <> btIfThenBlock And Program.Procedures(CurrentProcedure).Blocks(CurrentBlock).Type <> btIfElseBlock Then Err.Raise 10003, , "EndIf without If"
        Exit For
        
        
      ElseIf .Command.Index = cmdSetAlpha Then
        If GetColorDepth() < 24 Then Form1.Console.PrintText "Alpha Transparency requires at least 24Bit Color Depth to work properly!", &HCCCC&, ciWarning
      End If
      
      If .CommandArrayIndex <> -1 Then Program.Commands(.CommandArrayIndex) = .Command
    End With
  End If
  
  If Options.DebugMode Then If Options.ShowCurrentLineWhileParsing Then If Options.ParserDelay Then Common.Wait Options.ParserDelay
  
  
  If Program.State And psfPaused Then
    Form1.CodeEditor.SetArrow CurrentLine
    Common.Pause
  End If
  If Program.bCancel Then Exit For
Next

End Sub

Private Function ParseVariableDeclaration(Decl As String, CurrentProcedure As Long) As VariableInfo
Dim Pos As Long
Dim VarTypeName As String
Dim VarTypeIndex As Long
Dim W As Long
Pos = InStr(1, Decl, TypeSepChar)
If Pos Then
  VarTypeName = UCase$(Mid$(Decl, Pos + 1))
  VarTypeIndex = -1
  For W = 0 To Program.NumStructs - 1
    If UCase$(Program.Structs(W).Name) = VarTypeName Then
      VarTypeIndex = W
      Exit For
    End If
  Next
  If VarTypeIndex = -1 Then Err.Raise 100000, , "Type not defined"
  
  ParseVariableDeclaration.VarType = VarTypeIndex
  ParseVariableDeclaration.Name = Left$(Decl, Pos - 1)
Else
  ParseVariableDeclaration.VarType = -1
  ParseVariableDeclaration.Name = Decl
End If
End Function


Private Function IsValiedName(Name As String) As Boolean
If IsNumeric(Left$(Name, 1)) = False Then
  If InStr(1, Name, " ") = 0 Then
    If Left$(Name, 1) <> TypeSepChar Then
      If LineAnalyzer.GetCommandIndex(Name) = 0 Then
        If LineAnalyzer.GetOperandIndex(Name) = 0 Then
          IsValiedName = True
        End If
      End If
    End If
  End If
End If
End Function

Private Sub AddIndexParameter(C As Command, NewIndex As Long)
ReDim Preserve C.Parameters(0 To UBound(C.Parameters) + 1)
With C.Parameters(UBound(C.Parameters))
  .Type = ptBlockIndex
  .ValIndex = NewIndex
End With
End Sub

